# Copyright 2008 Canonical Ltd.  All rights reserved.

"""Tests of lazr.restful navigation."""

__metaclass__ = type

from pkg_resources import resource_filename
import os
import unittest

from van.testing.layer import zcml_layer
from zope.component import getMultiAdapter

from lazr.restful._resource import ReadWriteResource
from lazr.restful.error import expose, ClientErrorView, SystemErrorView
from lazr.restful.interfaces import IClientErrorView
from lazr.restful.testing.webservice import FakeRequest


class TestResource(ReadWriteResource):

    def __init__(self, callable, request):
        self.callable = callable
        super(TestResource, self).__init__(context=None, request=request)

    def do_GET(self):
        return self.callable()


class ErrorsTestCase(unittest.TestCase):

    def test_client_error_view_lookup(self):
        # An error decorated with lazr.restful.error.exposed will be adaptable
        # to IClientErrorView.
        request = FakeRequest(version='trunk')
        exception_view = getMultiAdapter(
            (expose(ValueError()), request), IClientErrorView)
        self.assertEquals(exception_view.__class__, ClientErrorView)

    def test_undecorated_exception(self):
        # If an undecorated exception (a regular Python exception) is
        # generated during a request the exception percolates all the way out
        # of the resource (to be handled by the publisher).

        def broken():
            raise RuntimeError('something broke')

        request = FakeRequest(version='trunk')
        resource = TestResource(broken, request)
        self.assertRaises(RuntimeError, resource)

    def test_decorated_exception(self):
        # If an undecorated exception is generated during a request the
        # SystemErrorView is used to generate the response.

        def broken():
            raise expose(RuntimeError('something broke'))

        request = FakeRequest(version='trunk')
        resource = TestResource(broken, request)
        result = resource()
        # The client error view sets the status code to 400 (client error).
        self.assertEqual(request.response.getStatus(), 400)
        # The message used to construct the exception is included in the body
        # of the response.
        self.assert_(result.startswith('something broke'))

    def test_passing_bad_things_to_expose(self):
        # The expose function only accepts instances of exceptions.  It
        # generates errors otherwise.
        self.assertRaises(ValueError, expose, 1)
        self.assertRaises(ValueError, expose, 'x')
        self.assertRaises(ValueError, expose, RuntimeError)


class FunctionalLayer:
    allow_teardown = False
    zcml = os.path.abspath(resource_filename('lazr.restful', 'ftesting.zcml'))


zcml_layer(FunctionalLayer)


def additional_tests():
    """See `zope.testing.testrunner`."""
    suite = unittest.TestLoader().loadTestsFromName(__name__)
    suite.layer = FunctionalLayer
    return suite
